using System;
using System.Collections.Generic;
using System.Text;
using NHMA = NHibernate.Mapping.Attributes;
using Iesi.Collections;

namespace infoMECHANICA.NHibernateORM.TestMappings
{
    /// <summary>
    /// NHibernate mappings of database Companies
    /// </summary>
    [NHMA.Class(Table = "Companies")]
    public partial class Company
    {
        private int _CompanyId;
        private string _Name;
        private Country _Country;
        private string _City;
        private ISet _Employees;


        /// <summary>
        /// Database entity of CompanyId column
        /// This is also unique identifier of Companies table
        /// </summary>
        [NHMA.Id(1, Name="CompanyId", Column = "CompanyId")]
        [NHMA.Generator(2, Class = "native")]
        public virtual int CompanyId
        {
            get { return this._CompanyId; }
            set { this._CompanyId = value; }		    
        }        
 
        /// <summary>
        /// Database entity of Name column
        /// </summary>
        [NHMA.Property(Column = "Name", NotNull=true)]
        public virtual string Name
        {
            get { return this._Name; }
            set { this._Name = value; }		    
        }        
 
        /// <summary>        
        /// Foreign key reference to Countries table
        /// </summary>        
        [NHMA.ManyToOne(Column = "CountryId", Lazy = NHMA.Laziness.Proxy, Class = "infoMECHANICA.NHibernateORM.TestMappings.Country, infoMECHANICA.NHibernateORM.TestMappings")]  
        public virtual Country Country        
        {
            get { return this._Country; }
            set { this._Country = value; }		    
        }        
 
        /// <summary>
        /// Database entity of City column
        /// </summary>
        [NHMA.Property(Column = "City", NotNull=true)]
        public virtual string City
        {
            get { return this._City; }
            set { this._City = value; }		    
        }        

        /// <summary>
        /// Database foreign key reference of relationship between Employees and Companies tables
        /// Lazy initialized Set of Employee dtos
        /// </summary>        
        [NHMA.Set(1, Table = "Employees", Lazy = true, Inverse = true, Cascade = NHMA.CascadeStyle.All)]
        [NHMA.Key(2, Column = "CompanyId")]
        [NHMA.OneToMany(3, Class = "infoMECHANICA.NHibernateORM.TestMappings.Employee, infoMECHANICA.NHibernateORM.TestMappings")]
        public virtual ISet Employees
        {
            get
            {
                if (_Employees == null)
                {
                    _Employees = new HashedSet();
                }
                return _Employees;
            }
            set
            {
                _Employees = value;
            }
        }        
    }
}
