using System;
using System.Collections.Generic;
using System.Text;
using NHMA = NHibernate.Mapping.Attributes;
using Iesi.Collections;

namespace infoMECHANICA.NHibernateORM.TestMappings
{
    /// <summary>
    /// NHibernate mappings of database Countries
    /// </summary>
    [NHMA.Class(Table = "Countries")]
    public partial class Country
    {
        private int _CountryId;
        private string _Name;
        private string _Region;
        private ISet _Companies;


        /// <summary>
        /// Database entity of CountryId column
        /// This is also unique identifier of Countries table
        /// </summary>
        [NHMA.Id(1, Name="CountryId", Column = "CountryId")]
        [NHMA.Generator(2, Class = "native")]
        public virtual int CountryId
        {
            get { return this._CountryId; }
            set { this._CountryId = value; }		    
        }        
 
        /// <summary>
        /// Database entity of Name column
        /// </summary>
        [NHMA.Property(Column = "Name", NotNull=true)]
        public virtual string Name
        {
            get { return this._Name; }
            set { this._Name = value; }		    
        }        
 
        /// <summary>
        /// Database entity of Region column
        /// </summary>
        [NHMA.Property(Column = "Region", NotNull=true)]
        public virtual string Region
        {
            get { return this._Region; }
            set { this._Region = value; }		    
        }        

        /// <summary>
        /// Database foreign key reference of relationship between Companies and Countries tables
        /// Lazy initialized Set of Company dtos
        /// </summary>        
        [NHMA.Set(1, Table = "Companies", Lazy = true, Inverse = true, Cascade = NHMA.CascadeStyle.All)]
        [NHMA.Key(2, Column = "CountryId")]
        [NHMA.OneToMany(3, Class = "infoMECHANICA.NHibernateORM.TestMappings.Company, infoMECHANICA.NHibernateORM.TestMappings")]
        public virtual ISet Companies
        {
            get
            {
                if (_Companies == null)
                {
                    _Companies = new HashedSet();
                }
                return _Companies;
            }
            set
            {
                _Companies = value;
            }
        }        
    }
}
