using System;
using infoMECHANICA.NHibernateORM.CriteriaBuilder;

namespace infoMECHANICA.NHibernateORM.TestMappings
{
    /// <summary>
    /// Recorder of Country class/Countries table
    /// </summary>
    [Serializable]
    public class CountryRec : CriteriaRecorder<CountryRec,Country>
    {
        private ICriterionProperty<int> _CountryId;
        private ICriterionProperty<string> _Name;
        private ICriterionProperty<string> _Region;
        private ISubcriteriaProperty<CompanyRec> _CompaniesByCountry;


        /// <summary>
        /// Recorder property for CountryId column (PRIMARY KEY!)
        /// </summary>
        public ICriterionProperty<int> CountryId
        {
            get 
            {
                if (this._CountryId == null)
                {
                    this._CountryId = new CriterionProperty<int>("CountryId", this);
                }
                
                return _CountryId; 
            }               
        }
        
        /// <summary>
        /// Gets the name of dto identifier property (when it's different than "Id")
        /// </summary>
        public override string GetIdPropertyName()
        {
            return "CountryId";
        }
 
        /// <summary>
        /// Recorder property for Name column
        /// </summary>
        public ICriterionProperty<string> Name
        {
            get 
            {
                if (this._Name == null)
                {
                    this._Name = new CriterionProperty<string>("Name", this);
                }
                
                return _Name; 
            }               
        }        
 
        /// <summary>
        /// Recorder property for Region column
        /// </summary>
        public ICriterionProperty<string> Region
        {
            get 
            {
                if (this._Region == null)
                {
                    this._Region = new CriterionProperty<string>("Region", this);
                }
                
                return _Region; 
            }               
        }        


        /// <summary>
        /// Recorder property of many-to-many relationship between Companies and Countries tables
        /// </summary>        
        public virtual ISubcriteriaProperty<CompanyRec> CompaniesByCountry
        {
            get
            {
                if (this._CompaniesByCountry == null)
                {
                    this._CompaniesByCountry = new SubcriteriaProperty<CompanyRec>("Country");
                }
                
                return _CompaniesByCountry;         
            }
        }        

        /// <summary>
        /// Constructor, creates the recorder with no parent
        /// </summary>
        public CountryRec()
        {
        }
        
        /// <summary>
        /// Constuctor, creates CriteriaRecorder and sets its parent
        /// </summary>
        /// <param name="parent">Parent of this recorder</param>
        public CountryRec (ICriteriaRecorder parent)
            : base(parent)
        {
        }
    }
}